package joystick;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.JFrame;
import javax.swing.UIDefaults;
import javax.swing.UIManager;

import test.TopicTest;

import dcps.Topic;
import dcps.TopicListener;
import dcps.TopicsScope;
import ethernet.Message;
import ethernet.MultiPeer;

public class JoystickClient {

    public static void main(String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }
    
    
    private static void createAndShowGUI() {
        System.out.println("Welcome Joystick DCPS Client.");
        final JFrame f = new JFrame("Joystick DCPS Client");
        
        updateLookAndFeel();
        f.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        f.addWindowListener(new WindowAdapter() {
        	public void windowClosing(WindowEvent e) {
                int confirmed = JOptionPane.showConfirmDialog(null,
                                "Are you sure you want to exit?", "User Confirmation",
                                JOptionPane.YES_NO_OPTION);
                if (confirmed == JOptionPane.YES_OPTION) {
                    f.dispose();
                    System.exit(0);
                }
        }
        });
        JoyPanel myPanel = new JoyPanel(f);
        f.add(myPanel);		
        f.pack();
        f.setSize(320,250);
	    Dimension dimScreen = java.awt.Toolkit.getDefaultToolkit().getScreenSize();  
		int posX = (dimScreen.width - 320) / 2;
		int posY = (dimScreen.height - 250) / 2;
		f.setLocation(posX, posY);
        f.setVisible(true);
    }
    
    private static void updateLookAndFeel() {        
	    try {
	        UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
	    } catch (Exception ex) {} 	    
    }
    
    
    public static class JoyPanel extends JPanel implements TopicListener{
		private static final long serialVersionUID = 1L;
		
		JoystickData joyData = new JoystickData();
    	long delay = 0;
    	long clockError = 0;
    	
    	private JTextField topicName = null;
    	private JButton startButton = null;
    	private JButton stopButton = null;
    	
    	private TopicsScope ts = new TopicsScope();
    	
    	public JoyPanel(JFrame frame) {
    		super(new BorderLayout());
    		JPanel drwPan = new DrawingPanel();
    		drwPan.setBounds(0, 50, 500, 640);
    		//setLayout(null);
            // Scroller Drawing Panel
            //JScrollPane scrollerDrawingPanel = new JScrollPane(drwPan);
            //scrollerDrawingPanel.setPreferredSize(new Dimension(200,200));
            //scrollerDrawingPanel.getHorizontalScrollBar().setUnitIncrement(20);
            //scrollerDrawingPanel.getVerticalScrollBar().setUnitIncrement(20);
            //add(scrollerDrawingPanel, BorderLayout.CENTER);
    		add(drwPan);
            
    		JPanel panelDlg = new JPanel();
    		panelDlg.setBounds(0, 0, 500, 40);
    		//panelDlg.setLayout(new BorderLayout());
    		add(panelDlg);
    		
    		
            topicName = new JTextField("T01");
            //topicName.addActionListener(this);
            panelDlg.add(topicName, BorderLayout.CENTER);
            topicName.setPreferredSize(new Dimension(60, 25));
            topicName.setEnabled(false);
            
            
    		startButton = new JButton("Start");
    		startButton.setEnabled(false);
    		startButton.addActionListener(new ActionListener() {
    			public void actionPerformed(ActionEvent e) {
    			    String s=new String(topicName.getText());
    			    try {
    		    		Topic t1 = ts.lookup(topicName.getText()); 		    	
    		    		t1.addListener(JoyPanel.this);	  
    		    		topicName.setEnabled(false);
    			        startButton.setEnabled(false);
    			        stopButton.setEnabled(true);
    			       
    			    } catch (Exception ex) {
    			            System.err.println("Joy: Error start.");
    			    }
    			}
    		});
    		startButton.setPreferredSize(new Dimension(80, 25));
    		panelDlg.add(startButton, BorderLayout.CENTER);
    		
    		
    		
    		stopButton = new JButton("Stop");
    		stopButton.setEnabled(true);
    		stopButton.addActionListener(new ActionListener() {
    			public void actionPerformed(ActionEvent e) {
    			    String s=new String(topicName.getText());
    			    try {
    			       ts.unSuscribe(topicName.getText());
    			       topicName.setEnabled(true);
     			       startButton.setEnabled(true);
     			       stopButton.setEnabled(false);
    			       
    			    } catch (Exception ex) {
    			            System.err.println("Joy: Error stop.");
    			    }
    			}
    		});
    		stopButton.setPreferredSize(new Dimension(80, 25));
    		panelDlg.add(stopButton, BorderLayout.CENTER);
            
    		Topic t1 = ts.lookup(topicName.getText()); 		    	
    		t1.addListener(JoyPanel.this);	
    		
    		
            joyData.setStickPosX(500);
            joyData.setStickPosY(500);
           
    		
    	}
    	
    	
        public class DrawingPanel extends JPanel {
    		private static final long serialVersionUID = 1L;
    		protected void paintComponent(Graphics g) {
                super.paintComponent(g);            
                Graphics2D g2d = (Graphics2D)g;

                //Clip graphics to the frame area.
                g2d.clipRect((-1)*this.getX(),(-1)*this.getY() , this.getWidth(), this.getHeight());
                //Set antialiasing for graphics and text
            	g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
            	g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON); 

            	int posX = 90;
            	int posY = 90;
            	
            	joyData.paint(g2d, posX, posY);
            	
            	g2d.setColor(Color.BLACK);
            	g2d.drawString("Msg. delay: "+delay+"us", 190, 15);
            	g2d.drawString("Clock error: "+clockError+"us", 190, 30);
                           
            }
        }


		@Override
		public void onMessage(Topic t, Message m) {
			joyData.fromString(new String(m.getBufferPayload()));
			delay = util.Clock.currentMicros() - m.getTimeStamp();
			clockError = util.Clock.getError();
			repaint();
		}
    }
    
    
}
