package joystick;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.Locale;

public class JoystickData {


	private boolean button0 = false;
	private boolean button1 = false;
	private boolean button2 = false;
	private boolean button3 = false;
	
	private int stickPosX = 0;
	private int stickPosY = 0;
	
	private float hat = (float) 0.0;
	
	public boolean isButton0() {
		return button0;
	}

	public void setButton0(boolean button0) {
		this.button0 = button0;
	}

	public boolean isButton1() {
		return button1;
	}

	public void setButton1(boolean button1) {
		this.button1 = button1;
	}

	public boolean isButton2() {
		return button2;
	}

	public void setButton2(boolean button2) {
		this.button2 = button2;
	}

	public boolean isButton3() {
		return button3;
	}

	public void setButton3(boolean button3) {
		this.button3 = button3;
	}

	public int getStickPosX() {
		return stickPosX;
	}

	public void setStickPosX(int stickPosX) {
		this.stickPosX = stickPosX;
	}

	public int getStickPosY() {
		return stickPosY;
	}

	public void setStickPosY(int stickPosY) {
		this.stickPosY = stickPosY;
	}

	public float getHat() {
		return hat;
	}

	public void setHat(float hat) {
		this.hat = hat;
	}
	
	JoystickData() {
		
	}
	
	public String toString() {
		String retval = new String("");	
		retval += button0; retval += ":";
		retval += button1; retval += ":";
		retval += button2; retval += ":";
		retval += button3; retval += ":";
		retval += String.format(Locale.ENGLISH,"%.2f", hat); retval += ":";
		retval += stickPosX; retval += ":";
		retval += stickPosY; 
		return retval;
	}
	
	public void fromString(String msg) {	
		String [] tokens = msg.split(":");
		if (tokens.length < 7) return;
		setButton0(Boolean.parseBoolean(tokens[0]));
		setButton1(Boolean.parseBoolean(tokens[1]));
		setButton2(Boolean.parseBoolean(tokens[2]));
		setButton3(Boolean.parseBoolean(tokens[3]));
		setHat((float)Double.parseDouble(tokens[4]));
		setStickPosX(Integer.parseInt(tokens[5]));
		setStickPosY(Integer.parseInt(tokens[6]));
	}
	
	public void paint(Graphics2D g2d, int xPos, int yPos) {
    	int posX = xPos;
    	int posY = yPos;
    	g2d.setColor(Color.BLACK);
    	g2d.drawOval(posX-2, posY-2, 4, 4);
    	
    	int diameter = 80;
    	g2d.drawOval(posX-diameter, posY-diameter, 2*diameter, 2*diameter);
    	
    	//double maxModule = Math.sqrt( 2 * Math.pow(500,2));
    	
    	double module = Math.sqrt(Math.pow(getStickPosX() - 500, 2) + Math.pow(getStickPosY() - 500, 2));
    	module = (module != 0) ? module : 1;
    	
    	double cosAlpha = Math.max(Math.abs(getStickPosX() - 500), Math.abs(getStickPosY() - 500)) / module;
    	//double senAlpha = Math.abs(joyData.getStickPosY() - 500) / module;
    	
    	double realModuleMax = 500;
    	if ( cosAlpha != 0 ) {
    	    realModuleMax = 500 / cosAlpha;
    	}
    	
    	double inCircleModule = diameter *  module / realModuleMax;
    	
    	int cordX = (int) (inCircleModule * (getStickPosX() - 500) / module);
    	int cordY = (int) (inCircleModule * (getStickPosY() - 500) / module);
    	g2d.drawLine(posX, posY, posX  + cordX, posY + cordY);
    	
    	if (isButton0()) g2d.setColor(Color.RED); else g2d.setColor(Color.GREEN);
    	g2d.fillRect(posX - 85, posY - 85, 10, 5);
    	if (isButton1()) g2d.setColor(Color.RED); else g2d.setColor(Color.GREEN);
    	g2d.fillRect(posX - 85, posY - 75, 10, 5);
    	if (isButton2()) g2d.setColor(Color.RED); else g2d.setColor(Color.GREEN);
    	g2d.fillRect(posX - 85, posY - 65, 10, 5);
    	if (isButton3()) g2d.setColor(Color.RED); else g2d.setColor(Color.GREEN);
    	g2d.fillRect(posX - 85, posY - 55, 10, 5);
    	
    	int hatX = xPos + 70;
    	int hatY = yPos - 75;
    	int hatSize = 10;
    	if (Math.abs(getHat() - 1) < 0.1) g2d.setColor(Color.RED); else g2d.setColor(Color.GREEN);
    	g2d.fillRect(hatX-hatSize, hatY, 5, 5);
    	if (Math.abs(getHat() - 0.13) < 0.1) g2d.setColor(Color.RED); else g2d.setColor(Color.GREEN);
    	g2d.fillRect(hatX-hatSize*2/3, hatY-hatSize*2/3, 5, 5);
    	if (Math.abs(getHat() - 0.25) < 0.1) g2d.setColor(Color.RED); else g2d.setColor(Color.GREEN);
    	g2d.fillRect(hatX, hatY-hatSize, 5, 5);
    	if (Math.abs(getHat() - 0.38) < 0.1) g2d.setColor(Color.RED); else g2d.setColor(Color.GREEN);
    	g2d.fillRect(hatX+hatSize*2/3, hatY-hatSize*2/3, 5, 5);
    	if (Math.abs(getHat() - 0.5) < 0.1) g2d.setColor(Color.RED); else g2d.setColor(Color.GREEN);
    	g2d.fillRect(hatX+hatSize, hatY, 5, 5);
    	if (Math.abs(getHat() - 0.63) < 0.1) g2d.setColor(Color.RED); else g2d.setColor(Color.GREEN);
    	g2d.fillRect(hatX+hatSize*2/3, hatY+hatSize*2/3, 5, 5);
    	if (Math.abs(getHat() - 0.75) < 0.1) g2d.setColor(Color.RED); else g2d.setColor(Color.GREEN);
    	g2d.fillRect(hatX, hatY+hatSize, 5, 5);
    	if (Math.abs(getHat() - 0.88) < 0.1) g2d.setColor(Color.RED); else g2d.setColor(Color.GREEN);
    	g2d.fillRect(hatX-hatSize*2/3, hatY+hatSize*2/3, 5, 5);
	}
}
