package dcps;

import java.util.ArrayList;
import java.util.HashSet;

import ethernet.Destination;
import ethernet.Message;
import ethernet.MessageTopicData;

/**
 * @author Jose Simo (c) ai2-UPV 2014
 *
 */
public class Topic {

	private String name = "";
	private ArrayList<TopicListener> listeners = null;
	private TopicsScope myScope = null;
	
	private HashSet<Destination> knownDestinations = new HashSet<Destination>();
	
	public String getName() {return name;}
	
	Topic(TopicsScope ts,String n) {
		name = n;
		myScope = ts;
		listeners = new ArrayList<TopicListener>();
	}
	
	public void addListener(TopicListener obj) {
		listeners.add(obj);
	}
	
	public void removeListener(TopicListener obj) {
		listeners.remove(obj);
	}
	
	public void addDestination(Destination obj) {
		boolean success = knownDestinations.add(obj);
		if (success) {
		     util.Log.logMessage(util.Log.LogType.INFO,"Adding destination in topic (suscribe): " 
		    		 						+ this.name + " dest: " + obj.getIPPort().toString());
		}
	}
	
	public void removeDestination(Destination obj) {
		boolean success = knownDestinations.remove(obj);
		if (success) {
			util.Log.logMessage(util.Log.LogType.INFO,"Removing destination in topic (unsuscribe): " 
											+ this.name + " dest: " + obj.getIPPort().toString());
		}
	}
	
	private Destination[] getDestinations() {
		return knownDestinations.toArray(new Destination[0]);
	}
	
	public boolean publish(byte[] payLoad) {
		boolean retval = true;
		Message m = new MessageTopicData(myScope.getMultiPeer().getLocalIPPort(), name,payLoad);
		
		/* false delay 
		util.Clock.Chronometer crono = new util.Clock.Chronometer();
		crono.start();
		while (crono.diffNow() < 5000) {}
		*/

		if (myScope != null) {
			Destination[] failedDest = myScope.getMultiPeer().send(getDestinations(), m);
			if (failedDest != null) {
				for (Destination dest: failedDest) {
					removeDestination(dest);
				}
			}
		}
		
		return retval;
	}
	
	public boolean callListeners(Message m) {
		boolean retval = true;		
		for (TopicListener listener: listeners) {		 
			listener.onMessage(this, m);
		}
		return retval;
	}
	 
}
