
package dcps;

import java.util.ArrayList;
import java.util.HashMap;

import ethernet.Destination;
import ethernet.IPPort;
import ethernet.Message;
import ethernet.MessageTopicData;
import ethernet.MessageTopicLinks;
import ethernet.MultiPeer;
import ethernet.MultiPeerListener;

/**
 * 
 * @author Jose Simo (c) ai2-UPV 2014
 *
 */
public class TopicsScope implements MultiPeerListener{

	private HashMap<String,Topic> topics = null;
	private MultiPeer netPeers = null;
	
	public TopicsScope(MultiPeer mp) {
		topics = new HashMap<String,Topic>();
		netPeers = (mp != null) ? mp : new MultiPeer();
		netPeers.addSpecificListener(Message.MessageType.TOPICDATA,this);
		netPeers.addSpecificListener(Message.MessageType.TOPICLINKS,this);
		askForSuscription();
	}
	
	public TopicsScope() {
		this(null);
	}
	
	public MultiPeer getMultiPeer() {return netPeers;}
	
	public Topic lookup(String topicName) {
		Topic topic = null;
		if (!topics.containsKey(topicName)) {
			topic = new Topic(this,topicName);
			topics.put(topicName, topic);
			askForSuscription();
			//notifySuscription(topicName); 
		} else {
			topic = topics.get(topicName);
		}
		return topic;
	}
	
	public boolean unSuscribe(String topicName) {
		Topic topic = topics.remove(topicName);	
		notifyUnSuscription(topicName);
		return topic != null;
	}
	/*
	private void notifySuscription(String name) { ////////////////
		ArrayList<String> tlist = new ArrayList<String>();
		tlist.add(name);
		MessageTopicLinks msg = new MessageTopicLinks(netPeers.getLocalIPPort(), MessageTopicLinks.CommandType.SUSCRIBE,tlist);
		netPeers.send(msg);
		//System.out.println("Sending suscription: ");
	}*/
	
	private void notifyUnSuscription(String name) { ////////////////
		ArrayList<String> tlist = new ArrayList<String>();
		tlist.add(name);
		MessageTopicLinks msg = new MessageTopicLinks(netPeers.getLocalIPPort(), MessageTopicLinks.CommandType.UNSUSCRIBE,tlist);
		netPeers.send(msg);
	}
	
	private void askForSuscription() { /////
		//Ask to ALL peers to their list of registered topics. Local Peer included.
		MessageTopicLinks msgTL = new MessageTopicLinks(netPeers.getLocalIPPort(), MessageTopicLinks.CommandType.ASKTOPICS,null);
		netPeers.send(msgTL);
	}
	
	@Override
	public void onNewConnection(Destination dest) {
		//System.out.println("TopicsScope: New connection: ");
		askForSuscription();
	}
	
	@Override
	public void onMessage(Message m) {
		if (m.getType() == Message.MessageType.TOPICDATA) {
			//Only propagate data to already registered topics.
			String name = ((MessageTopicData)m).getTopicName();
			Topic topic = topics.get(name);	
			if (topic != null) topic.callListeners(m); 
		}
		if (m.getType() == Message.MessageType.TOPICLINKS) { 
			MessageTopicLinks msgTL = (MessageTopicLinks)m;
			ArrayList<String> tlist = null;
			switch (msgTL.getCommand()) {
				case ASKTOPICS:
					tlist = new ArrayList<String>();
					for (Topic topic: topics.values()) {
						tlist.add(topic.getName());
					}
					MessageTopicLinks msgToSend = new MessageTopicLinks(netPeers.getLocalIPPort(), MessageTopicLinks.CommandType.SUSCRIBE,tlist);
					this.getMultiPeer().send(msgToSend);					
					break;
				case SUSCRIBE:
					tlist = msgTL.getTopics();
					IPPort srcIPP = msgTL.getSourceIPPort();
					Destination srcDest = this.getMultiPeer().getDestinationByIPPort(srcIPP);
					if (srcDest != null) {
						for (int i=0; i<tlist.size();i++) {	
							Topic topic = topics.get(tlist.get(i));
							if (topic != null) {
								topic.addDestination(srcDest);
							}
						}
					}
					break;
				case UNSUSCRIBE:
					tlist = msgTL.getTopics();
					srcIPP = msgTL.getSourceIPPort();
					srcDest = this.getMultiPeer().getDestinationByIPPort(srcIPP);
					if (srcDest != null) {
						for (int i=0; i<tlist.size();i++) {	
							Topic topic = topics.get(tlist.get(i));
							if (topic != null) {
								topic.removeDestination(srcDest);
							}
						}
					}
					break;
			}
		}
	}
	
}
