
package ethernet;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * This Message extension handles messages with any pre-defined structure.
 * The header is empty.
 * 
 * @author Jose Simo (c) ai2-UPV 2014
 */
public class MessagePlain extends Message {

	/**
	 * Create a plain message based on the passed buffer.
	 * @param buf message content
	 */
	public MessagePlain(IPPort srcIPP, byte[] buf) {
		bufferHeader = null;
		bufferPayload = buf;
		timeStamp = util.Clock.currentMicros();
		type = Message.MessageType.PLAIN;
		sourceIPPort = srcIPP;
		bufferPrologue = computePrologue();
	}
	
	/**
	 * Creates an empty message to be populated using the 
	 * "fromInputStream" instance method.
	 */
	public MessagePlain() {
		this(null,null);
	}

	/**
	 * Populates a plan message reading data (after the prologue) from an input stream.
	 */
	@Override
	protected boolean fromInputStream(InputStream is, int size) throws IOException {
		boolean retval = true;
		int nRead = 0;
		byte[] data = new byte[size];
		nRead = is.read(data, 0, data.length);
		if (nRead == -1) retval = false;
		bufferPayload = data;
		bufferHeader = null;
		return retval;
	}
	
	@Override
	protected void toOutputStream(OutputStream os) throws IOException {
		//write the message prologue (if it exists)
		if (bufferPrologue != null) {
			os.write(bufferPrologue, 0, bufferPrologue.length);
		}
		//write the message payload (if it exists)
		if (bufferPayload != null) {
			os.write(bufferPayload, 0, bufferPayload.length);
		}
	}
}
