
package ethernet;

import java.util.ArrayList;

/**
 * Synchronized message queue.<p>
 * Policy: FIFO.<p>
 * The "pull" operation blocks the calling thread if the queue is empty.<p>
 * 
 * @author Jose Simo (c) ai2-UPV 2014
 */
public class QueueMessage {
	
	private ArrayList<Message> queue = new ArrayList<Message>();
	/**
	 * Adds a message to the queue
	 * @param m
	 */
	public synchronized void push(Message m) {
		queue.add(m);	
		notify();
	}

	/**
	 * Gets the queue size
	 * @return integer
	 */
	public synchronized int size() {
		return queue.size();
	}
	
	/**
	 * Gets a message from the head of the queue following the FIFO policy.
	 * If the queue is empty, this call blocks the calling thread until 
	 * a message is available.
	 * @return a Message.
	 */
	public synchronized Message pull() {
		Message retval = null;
		while (queue.size() <= 0)
			try {
				wait();
			} catch (InterruptedException e) {
				util.Log.logMessage(util.Log.LogType.ERROR,"MessageQueue: Wait interrupted.");
				e.printStackTrace();
			}
		retval = queue.remove(0);
		return retval;
	}
	
}
