
package util;

import java.net.Inet6Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;

/**
 * Network utilities.
 * The main purpose of this singleton is to provide a controlled way to
 * get the local IP.
 * 
 * @author Jose Simo (c) ai2-UPV 2014
 */
public class Net {

	private static ArrayList<InetAddress> addrs = null;
	
	public static String getLocalHostIP() {
		String addr = null;
		try {
			addr = InetAddress.getLocalHost().getHostAddress();
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
		return addr;
	}
	
	public static String getFavouriteLocalIP() {	
		if (addrs == null) {
			try {
				addrs = getLocalIPsList();
			} catch (SocketException e) {
				e.printStackTrace();
			}
		}
		String addr = addrs.size() > 0 ? addrs.get(addrs.size()-1).getHostAddress() : "127.0.0.1";
		return addr;
	}
	
	private static ArrayList<InetAddress> getLocalIPsList() throws SocketException{
		ArrayList<InetAddress> retval = new ArrayList<InetAddress>();
		Enumeration<NetworkInterface> interfaces = NetworkInterface.getNetworkInterfaces();
		for (NetworkInterface interface_ : Collections.list(interfaces)) {
		  // we shouldn't care about loopback addresses
		  if (interface_.isLoopback()) { continue;}
		  if (!interface_.isUp()) { continue; }
		  Enumeration<InetAddress> addresses = interface_.getInetAddresses();
		  for (InetAddress address : Collections.list(addresses)) {
			    if (address instanceof Inet6Address) {
			        continue;
			    } else {
			    	retval.add(address);
			    }
		  }
		}
		return retval;
	}
	
	public static long ipAddrAsLong(InetAddress addr) {
		byte[] addrBuff = addr.getAddress();
		byte[] longBuff = new byte[8];
		longBuff[0] = addrBuff[3];
		longBuff[1] = addrBuff[2];
		longBuff[2] = addrBuff[1];
		longBuff[3] = addrBuff[0];
		longBuff[4] = 0; longBuff[5] = 0; longBuff[6] = 0; longBuff[7] = 0;	
		long retval = util.Bin.qWordToLong(longBuff);
		return retval;
	}
	
	/*
	public static void main(String[] args)  {
		
		ArrayList<InetAddress> addrs = null;
		try {
			addrs = getLocalIPsList();
		} catch (SocketException e) {
			e.printStackTrace();
		}
		if (addrs != null)
			for (int i=0; i<addrs.size(); i++) {
				System.out.println("Address: "+ addrs.get(i).getHostAddress());
			}
		
	}*/
}
